# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 CERN.
# Copyright (C) 2021 Northwestern University.
#
# Invenio-RDM-Records is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Celery tasks for fixtures."""


from celery import shared_task
from invenio_access.permissions import system_identity
from invenio_pidstore.errors import PIDAlreadyExists

from ..proxies import current_communities


@shared_task
def create_demo_community(data):
    """Create a demo community."""
    service = current_communities.service
    try:
        service.create(data=data, identity=system_identity)
    except PIDAlreadyExists:
        pass



# # -*- coding: utf-8 -*-
# #
# # This file is part of Invenio.
# # Copyright (C) 2020 CERN.
# #
# # Invenio is free software; you can redistribute it and/or modify it
# # under the terms of the MIT License; see LICENSE file for more details.

# """Community module tests."""

import pytest
from flask import url_for
from invenio_accounts.testutils import login_user_via_session
from six.moves.urllib.parse import urljoin


@pytest.mark.skip()
def test_simple_flow(
        db, es_clear, community, client,
        community_owner, users):
    """Test basic operations on members."""
    comid, community = community
    new_member = users[0]
    login_user_via_session(client, user=new_member)
    community_members_list_url = url_for(
        'invenio_communities_members.community_members_api',
        pid_value=comid.pid_value)

    resp = client.post(community_members_list_url)
    assert resp.status_code == 201
    community_member_id = resp.json['id']

    login_user_via_session(client, user=community_owner)
    community_member_url = url_for(
        'invenio_communities_members.community_requests_api',
        pid_value=comid.pid_value,
        membership_id=community_member_id)

    resp = client.get(community_member_url)
    assert resp.status_code == 200
    membership_links = resp.json['links']

    resp = client.post(membership_links['comment'],  json={
        'message': 'Hello there'
    })
    assert resp.status_code == 200

    login_user_via_session(client, user=new_member)
    resp = client.post(membership_links['comment'],  json={
        'message': 'Oh hi Mark'
    })
    assert resp.status_code == 200

    login_user_via_session(client, user=community_owner)
    resp = client.post(membership_links['accept'], json={
        'message': 'Welcome aboard. Im removing you soon', 'role': 'curator'
    })
    assert resp.status_code == 200
    assert resp.json['role'] == 'curator'

    resp = client.put(community_member_url,  json={
        'role': 'admin'
    })
    assert resp.status_code == 200
    assert resp.json['role'] == 'admin'

    resp = client.post(membership_links['comment'],  json={
        'message': 'Goodbye'
    })
    assert resp.status_code == 200

    resp = client.delete(community_member_url)
    assert resp == 204


@pytest.mark.skip()
def test_alternate_flow(
        db, es_clear, community, client,
        community_owner, users):
    """Test basic operations with the alternate flow on members."""
    comid, community = community
    new_member = users[0]
    login_user_via_session(client, user=community_owner)
    community_members_list_url = url_for(
        'invenio_communities_members.community_members_api',
        pid_value=comid.pid_value)

    email = 'abc@test.com'
    resp = client.post(community_members_list_url, json={
        'role': 'curator', 'email': email
    })
    assert resp.status_code == 201
    community_member_id = resp.json['id']

    token = create_token(email).decode("utf-8")
    login_user_via_session(client, user=new_member)

    community_member_url = url_for(
        'invenio_communities_members.community_requests_api',
        pid_value=comid.pid_value,
        membership_id=community_member_id)

    resp = client.get(urljoin(community_member_url, '?token={}'.format(token)))
    assert resp.status_code == 200
    membership_links = resp.json['links']
    resp = client.post(
        urljoin(membership_links['comment'], '?token={}'.format(token)),
        json={
            'message': 'Oh hi Mark',
        })
    assert resp.status_code == 200

    login_user_via_session(client, user=community_owner)
    resp = client.post(membership_links['comment'],  json={
        'message': 'Hello there'
    })
    assert resp.status_code == 200

    login_user_via_session(client, user=new_member)
    resp = client.post(
        membership_links['accept'],
        json={
            'message': 'I shall accept',
            'token': token
        })
    assert resp.status_code == 200
    resp = client.post(
        membership_links['reject'],
        json={
            'message': 'I changed my mind',
            'token': token
        })
    assert resp.status_code == 404

    resp = client.delete(community_member_url)
    assert resp.status_code == 204



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2020 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.
"""Utilities."""

# from __future__ import absolute_import, print_function

# from datetime import timedelta
# from functools import partial

# from flask import current_app, render_template, request
# from flask_babelex import gettext as _
# from flask_mail import Message
# from invenio_accounts.models import User
# from invenio_base.utils import obj_or_import_string
# from invenio_mail.tasks import send_email
# from invenio_pidstore.resolver import Resolver
# from invenio_records.api import Record
# from invenio_records_rest.utils import LazyPIDValue
# from six.moves.urllib.parse import parse_qs, urlencode, urlsplit, urlunsplit
# from werkzeug.routing import BaseConverter
# from werkzeug.utils import cached_property

# # from .members.api import CommunityMember
# # from .members.models import CommunityMemberRole, CommunityMemberStatus


# def format_url_template(url_template, absolute=True, **kwargs):
#     """Formats a URL template-like string."""
#     scheme, netloc, path, query, fragment = urlsplit(url_template)
#     scheme = scheme or request.scheme
#     netloc = netloc or request.host

#     # TODO: Find a more specific exception to raise here (or custom)
#     if absolute:
#         assert netloc
#     path = path.format(**kwargs)
#     query = urlencode({
#         k: [v.format(**kwargs) for v in vals]
#         for k, vals in parse_qs(query).items()
#     }, doseq=True)
#     return urlunsplit((scheme, netloc, path, query, fragment))


# def send_invitation_email(membership, recipients, community, token=b''):
#     """Send email notification for a member invitation or request."""
#     msg = Message(
#         _("Community membership invitation"),
#         sender=current_app.config.get('SUPPORT_EMAIL'),
#         recipients=recipients,
#     )
#     template = ("member_invitation_email.tpl" if membership.request.is_invite
#                 else "member_request_email.tpl")
#     msg.body = render_template(
#         "invenio_communities/email/{}".format(template),
#         community=community,
#         days=timedelta(
#             seconds=current_app.config[
#                 "COMMUNITIES_MEMBERSHIP_REQUESTS_CONFIRMLINK_EXPIRES_IN"]
#         ).days,
#         membership=membership,
#         invitation_link=format_url_template(
#             current_app.config.get(
#                 'COMMUNITIES_INVITATION_LINK_URL',
#                 '/communities/{com_id}/members/requests/{mem_id}?token={token}'
#             ),
#             mem_id=str(membership.id),
#             com_id=community["id"],
#             token=token.decode("utf-8")
#         ),
#         community_members_link=format_url_template(
#             '/communities/{com_id}/members?tab=pending',
#             com_id=community["id"]
#         )
#     )
#     send_email.delay(msg.__dict__)


# def send_inclusion_email(community_record, recipients):
#     """Send email notification for a record invitation or request."""
#     msg = Message(
#         _("Community record inclusion"),
#         sender=current_app.config.get('SUPPORT_EMAIL'),
#         recipients=recipients,
#     )
#     template = ("record_invitation_email.tpl" if
#                 community_record.request.is_invite
#                 else "record_request_email.tpl")
#     msg.body = render_template(
#         "invenio_communities/email/{}".format(template),
#         community_record=community_record,
#         record_invitation_link=format_url_template(
#             current_app.config.get(
#                 'COMMUNITIES_RECORD_INVITATION_LINK_URL',
#                 '/records/{recid}'
#             ),
#             recid=community_record.record.pid.pid_value
#         ),
#         # TODO: make overridable
#         community_curation_link=format_url_template(
#             '/communities/{com_id}/curate?q=',
#             com_id=community_record.community["id"]
#         )
#     )
#     send_email.delay(msg.__dict__)


# # TODO: Remove when
# # https://github.com/inveniosoftware/invenio-records-rest/pull/294 is merged
# class LazyPIDConverter(BaseConverter):
#     """Converter for PID values in the route mapping."""

#     def __init__(self, url_map, pid_type, getter=None, record_class=None,
#                  object_type='rec'):
#         """Initialize the converter."""
#         super(LazyPIDConverter, self).__init__(url_map)
#         self.pid_type = pid_type
#         self.getter = getter
#         self.record_class = record_class
#         self.object_type = object_type

#     @cached_property
#     def resolver(self):
#         """PID resolver."""
#         record_cls = obj_or_import_string(
#             self.record_class, default=Record)
#         getter = obj_or_import_string(
#             self.getter,
#             default=partial(record_cls.get_record, with_deleted=True))
#         return Resolver(
#             pid_type=self.pid_type,
#             object_type=self.object_type,
#             getter=getter
#         )

#     def to_python(self, value):
#         """Resolve PID value."""
#         return LazyPIDValue(self.resolver, value)


# comid_url_converter = (
#     'lazy_pid(comid,'
#     'record_class="invenio_communities:Community",'
#     'object_type="com")'
# )


# # def set_default_admin(community):
# #     """Set the Admin in a newly created community."""
# #     user = User.query.get(community['created_by'])
# #     CommunityMember.create(
# #         community=community,
# #         role=CommunityMemberRole.ADMIN,
# #         user=user,
# #         status=CommunityMemberStatus.ACCEPTED
# #     )



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
# Copyright (C) 2020 Northwestern University.
#
# Invenio-Records-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Invenio Communities Service API config."""

from invenio_records_resources.services import FileServiceConfig
from invenio_records_resources.services.files.links import FileLink
from invenio_records_resources.services.records.components import \
    DataComponent, MetadataComponent
from invenio_records_resources.services.records.config import \
    RecordServiceConfig
from invenio_records_resources.services.records.config import \
    SearchOptions as SearchOptionsBase
from invenio_records_resources.services.records.links import RecordLink, \
    pagination_links
from invenio_records_resources.services.records.search import terms_filter

from invenio_communities.communities.records.api import Community

from ..schema import CommunitySchema
from .components import CommunityAccessComponent, PIDComponent
from .permissions import CommunityPermissionPolicy


class SearchOptions(SearchOptionsBase):
    """Search options."""

    facets_options = dict(
        aggs={
            'type': {
                'terms': {'field': 'metadata.type'},
            },
            'domain': {
                'terms': {'field': 'metadata.domains'},
            },
        },
        post_filters={
            'type': terms_filter('metadata.type'),
            'domain': terms_filter('metadata.domains'),
        }
    )


class CommunityServiceConfig(RecordServiceConfig):
    """Communities service configuration."""

    # Common configuration
    permission_policy_cls = CommunityPermissionPolicy

    # Record specific configuration
    record_cls = Community

    # Search configuration
    search = SearchOptions

    # Service schema
    schema = CommunitySchema

    links_item = {
        "self": RecordLink("{+api}/communities/{id}"),
        "self_html": RecordLink("{+ui}/communities/{id}"),
        "settings_html": RecordLink("{+ui}/communities/{id}/settings"),
        "logo": RecordLink("{+api}/communities/{id}/logo"),
        "rename": RecordLink("{+api}/communities/{id}/rename")
    }

    links_search = pagination_links("{+api}/communities{?args*}")
    links_user_search = pagination_links("{+api}/user/communities{?args*}")

    # Service components
    components = [
        MetadataComponent,
        PIDComponent,
        CommunityAccessComponent,
    ]


class CommunityFileServiceConfig(FileServiceConfig):
    """Configuration for community files."""

    record_cls = Community
    permission_policy_cls = CommunityPermissionPolicy

    file_links_item = {
        "self": FileLink("{+api}/communities/{id}/logo"),
    }



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2021 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Records API."""

from __future__ import absolute_import, print_function

from invenio_records.systemfields import ConstantField, ModelField
from invenio_records_resources.records.api import FileRecord, Record
from invenio_records_resources.records.systemfields import FilesField, \
    IndexField, PIDField
from werkzeug.local import LocalProxy

from . import models
from .providers import CommunitiesIdProvider
from .systemfields.access import CommunityAccessField


class CommunityFile(FileRecord):
    """Community file API."""

    model_cls = models.CommunityFileMetadata
    record_cls = LocalProxy(lambda: Community)


class Community(Record):
    """Community API."""

    pid = PIDField('id', provider=CommunitiesIdProvider, create=False)
    schema = ConstantField(
        '$schema', 'local://communities/communities-v1.0.0.json')

    model_cls = models.CommunityMetadata

    index = IndexField(
        "communities-communities-v1.0.0",
        search_alias="communities"
    )

    access = CommunityAccessField()

    bucket_id = ModelField(dump=False)
    bucket = ModelField(dump=False)
    files = FilesField(
        store=False,
        file_cls=CommunityFile,
        # Don't delete, we'll manage in the service
        delete=False,
    )



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
# Copyright (C) 2021 TU Wien.
#
# Invenio-RDM-Records is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""API views."""

from flask import Blueprint


def create_communities_api_blueprint(app):
    """Create communities api blueprint."""
    ext = app.extensions["invenio-communities"]
    # control blueprint endpoints registration
    if app.config["COMMUNITIES_ENABLED"]:
        return ext.communities_resource.as_blueprint()
    else:
        # return dummy blueprint
        return Blueprint(
            "invenio_communities_api",
            __name__,
        )



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2021 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Pytest configuration."""

import pytest


@pytest.fixture(scope='module')
def celery_config():
    """Override pytest-invenio fixture."""
    return {}

@pytest.fixture(scope='module')
def app_config(app_config):
    """Override pytest-invenio app_config fixture."""
    app_config['RECORDS_REFRESOLVER_CLS'] = \
        "invenio_records.resolver.InvenioRefResolver"
    app_config['RECORDS_REFRESOLVER_STORE'] = \
        "invenio_jsonschemas.proxies.current_refresolver_store"

    return app_config



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
#
# Invenio-Records-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Community schema."""

from flask import current_app
from flask_babelex import lazy_gettext as _
from invenio_rdm_records.services.schemas.metadata import AffiliationSchema, \
    FundingSchema
from invenio_rdm_records.services.schemas.parent.access import Agent
from invenio_records_resources.services.records.schema import BaseRecordSchema
from marshmallow import Schema, fields, missing, validate
from marshmallow_utils.fields import NestedAttribute, SanitizedHTML, \
    SanitizedUnicode


def _not_blank(**kwargs):
    """Returns a non-blank validation rule."""
    max_ = kwargs.get('max','')
    return validate.Length(error=_(f'Not empty string and less than {max_} characters allowed.'), min=1, **kwargs)


class CommunityAccessSchema(Schema):

    owned_by = fields.List(fields.Nested(Agent))

    visibility = fields.Str(validate=validate.OneOf([
        'public',
        'restricted',
    ]))
    member_policy = fields.Str(validate=validate.OneOf([
        'open',
        'closed',
    ]))
    record_policy = fields.Str(validate=validate.OneOf([
        'open',
        'closed',
        'restricted',
    ]))


class CommunityMetadataSchema(Schema):
    """Community metadata schema."""

    COMMUNITY_TYPES = [
        'organization',
        'event',
        'topic',
        'project',
    ]

    title = SanitizedUnicode(required=True, validate=_not_blank(max=250))
    description = SanitizedUnicode(validate=_not_blank(max=2000))

    curation_policy = SanitizedHTML(validate=_not_blank(max=2000))
    page = SanitizedHTML(validate=_not_blank(max=2000))

    # TODO: Use general small vocabularies
    type = SanitizedUnicode(
        validate=validate.OneOf(COMMUNITY_TYPES)
    )
    website = fields.Url(validate=_not_blank())
    funding = fields.List(fields.Nested(FundingSchema))
    organizations = fields.List(fields.Nested(AffiliationSchema))

    # TODO: Add when general vocabularies are ready
    # domains = fields.List(fields.Str())


class CommunitySchema(BaseRecordSchema):
    """Schema for the community metadata."""

    id = SanitizedUnicode(validate=_not_blank(max=100))
    metadata = NestedAttribute(CommunityMetadataSchema, required=True)
    access = NestedAttribute(CommunityAccessSchema, required=True)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2021 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Community access system field."""

from invenio_rdm_records.records.systemfields.access import Owner, Owners, \
    RecordAccessField


class _Owner(Owner):

    @property
    def user(self):
        if self.owner_type == 'user':
            return self.owner_id


class _Owners(Owners):

    owner_cls = _Owner

    def __init__(self, owners=None, owner_cls=None):
        """Create a new list of owners."""
        self.owner_cls = owner_cls or self.owner_cls
        for owner in owners or []:
            self.add(owner)


class CommunityAccess:
    """Access management per community."""

    VISIBILITY_LEVELS = ('public', 'restricted')
    MEMBER_POLICY_LEVELS = ('open', 'closed')
    RECORD_POLICY_LEVELS = ('open', 'closed', 'restricted')

    owners_cls = _Owners

    def __init__(
        self,
        visibility=None,
        member_policy=None,
        record_policy=None,
        owned_by=None,
        owners_cls=None,
    ):
        """Create a new CommunityAccess object.

        If ``owned_by`` is not specified, a new instance of ``owners_cls``
        will be used.

        :param visibility: The visibility level.
        :param owned_by: The set of community owners
        """
        self.visibility = visibility or 'public'
        self.member_policy = member_policy or 'open'
        self.record_policy = record_policy or 'open'

        owners_cls = owners_cls or self.owners_cls
        self.owned_by = owned_by if owned_by else owners_cls()

        self.errors = []

    def _validate_visibility_level(self, level):
        return level in self.VISIBILITY_LEVELS

    def _validate_member_policy_level(self, level):
        return level in self.MEMBER_POLICY_LEVELS

    def _validate_record_policy_level(self, level):
        return level in self.RECORD_POLICY_LEVELS

    @property
    def visibility(self):
        """Get the visibility level."""
        return self._visibility

    @visibility.setter
    def visibility(self, value):
        """Set the visibility level."""
        if not self._validate_visibility_level(value):
            raise ValueError(f"Unknown visibility level: {value}")
        self._visibility = value

    @property
    def member_policy(self):
        """Get the member policy level."""
        return self._member_policy

    @member_policy.setter
    def member_policy(self, value):
        """Set the member policy level."""
        if not self._validate_member_policy_level(value):
            raise ValueError(f"Unknown member policy level: {value}")
        self._member_policy = value

    @property
    def record_policy(self):
        """Get the record policy level."""
        return self._record_policy

    @record_policy.setter
    def record_policy(self, value):
        """Set the record policy level."""
        if not self._validate_record_policy_level(value):
            raise ValueError(f"Unknown record policy level: {value}")
        self._record_policy = value

    def dump(self):
        """Dump the field values as dictionary."""
        return {
            "visibility": self.visibility,
            "member_policy": self.member_policy,
            "record_policy": self.record_policy,
            "owned_by": self.owned_by.dump(),
        }

    def refresh_from_dict(self, access_dict):
        """Re-initialize the Access object with the data in the access_dict."""
        new_access = self.from_dict(access_dict)
        self.visibility = new_access.visibility
        self.member_policy = new_access.member_policy
        self.record_policy = new_access.record_policy
        self.owned_by = new_access.owned_by

    @classmethod
    def from_dict(
        cls,
        access_dict,
        owners_cls=None,
    ):
        """Create a new Access object from the specified 'access' property.

        The new ``CommunityAccess`` object will be populated with new instances
        from the configured classes.
        If ``access_dict`` is empty, the ``Access`` object will be populated
        with new instance ``owners_cls``.
        """
        owners_cls = owners_cls or cls.owners_cls

        errors = []

        # provide defaults in case there is no 'access' property
        owned_by = owners_cls()

        if access_dict:
            for owner_dict in access_dict.get("owned_by", []):
                try:
                    owned_by.add(owned_by.owner_cls(owner_dict))
                except Exception as e:
                    errors.append(e)

        access = cls(
            visibility=access_dict.get("visibility"),
            member_policy=access_dict.get("member_policy"),
            record_policy=access_dict.get("record_policy"),
            owned_by=owned_by
        )
        access.errors = errors
        return access

    def __repr__(self):
        """Return repr(self)."""
        return (
            "<{} (visibility: {}, "
            "member_policy: {}, "
            "record_policy: {}, "
            "owners: {})>"
        ).format(
            type(self).__name__,
            self.visibility,
            self.member_policy,
            self.record_policy,
            self.owned_by,
        )


class CommunityAccessField(RecordAccessField):
    """System field for managing community access."""

    def __init__(self, *args, access_obj_class=CommunityAccess, **kwargs):
        """Create a new CommunityAccessField instance."""
        super().__init__(*args, access_obj_class=access_obj_class, **kwargs)

    def obj(self, instance):
        """Get the access object."""
        obj = self._get_cache(instance)
        if obj is not None:
            return obj

        data = self.get_dictkey(instance)
        if data:
            obj = self._access_obj_class.from_dict(data)
        else:
            obj = self._access_obj_class()

        self._set_cache(instance, obj)
        return obj

    # NOTE: The original RecordAccessField dumps some non-existing fields
    def post_dump(self, *args, **kwargs):
        """Called before a record is dumped."""
        pass

    def pre_load(self, *args, **kwargs):
        """Called before a record is dumped."""
        pass



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 CERN.
#
# Invenio-Records-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Community views."""

from .api import create_communities_api_blueprint
from .ui import create_ui_blueprint

__all__ = (
    'create_communities_api_blueprint',
    'create_ui_blueprint'
)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 CERN.
#
# Invenio-Records-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Community views."""

from .config import CommunityResourceConfig
from .resource import CommunityResource

__all__ = (
    'CommunityResource',
    'CommunityResourceConfig',
)



# # -*- coding: utf-8 -*-
# #
# # This file is part of Invenio.
# # Copyright (C) 2020 CERN.
# #
# # Invenio is free software; you can redistribute it and/or modify it
# # under the terms of the MIT License; see LICENSE file for more details.

# """Proxy definitions."""

# from __future__ import absolute_import, print_function

# from flask import current_app
# from invenio_rest.errors import RESTException
# from itsdangerous import BadData, JSONWebSignatureSerializer, SignatureExpired


# class InvalidTokenError(RESTException):
#     """Resource access token for invalid token."""

#     code = 400
#     description = 'The token is invalid.'


# def create_token(payload):
#     """Create a token referencing the object id with extra data.

#     Note random data is added to ensure that no two tokens are identical.
#     """
#     jss = JSONWebSignatureSerializer(current_app.config['SECRET_KEY'])
#     return jss.dumps(payload)


# def validate_token(token, expected_value=None):
#     """Validate secret link token.

#     :param token: Token value.
#     :param expected_value: A dictionary of key/values that must be present
#         in the data part of the token (i.e. included via ``extra_data`` in
#         ``create_token``).
#     """
#     try:
#         # Load token and remove random data.
#         data = load_token(token)

#         # Compare expected data with data in token.
#         if expected_value:
#             if expected_value != data:
#                 return None
#         return data
#     except BadData:
#         raise InvalidTokenError()


# def load_token(token, force=False):
#     """Load data in a token.

#     :param token: Token to load.
#     :param force: Load token data even if signature expired.
#                     Default: False.
#     """
#     jss = JSONWebSignatureSerializer(current_app.config['SECRET_KEY'])

#     try:
#         data = jss.loads(token)
#     except SignatureExpired as e:
#         if not force:
#             raise
#         data = e.payload

#     return data



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2020-2021 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Community permissions."""

from elasticsearch_dsl.query import Q
from flask_principal import UserNeed
from invenio_access.permissions import any_user
from invenio_rdm_records.services.generators import \
    IfRestricted as IfRestrictedBase
from invenio_records_permissions.generators import AnyUser, \
    AuthenticatedUser, Generator, SystemProcess
from invenio_records_permissions.policies import BasePermissionPolicy


class IfRestricted(IfRestrictedBase):
    """IfRestricted.

    IfRestricted(
        'record',
        then_=[RecordPermissionLevel('view')],
        else_=[ActionNeed(superuser-access)],
    )

    A record permission level defines an aggregated set of
    low-level permissions,
    that grants increasing level of permissions to a record.

    """

    def generators(self, record):
        """Get the "then" or "else" generators."""
        if record is None:
            # TODO - when permissions on links are checked, the record is not
            # passes properly, causing below ``record.access`` to fail.
            return self.else_
        is_restricted = getattr(
            record.access, self.field, "restricted")
        return self.then_ if is_restricted == "restricted" else self.else_


class CommunityOwners(Generator):
    """Allows community owners."""

    def needs(self, record=None, **kwargs):
        """Enabling Needs."""
        if record:
            return [
                UserNeed(owner.owner_id)
                for owner in record.access.owned_by
                ]
        else:
            return [any_user]

    def query_filter(self, identity=None, **kwargs):
        """Filters for current identity as owner."""
        users = [n.value for n in identity.provides if n.method == "id"]
        return Q(
            "terms",
            **{"access.owned_by.user": users}
            )


class CommunityPermissionPolicy(BasePermissionPolicy):
    """Permissions for Community CRUD operations."""

    can_create = [AuthenticatedUser(), SystemProcess()]

    can_read = [
        IfRestricted(
            'visibility',
            then_=[CommunityOwners()],
            else_=[AnyUser()]),
        ]

    can_update = [CommunityOwners(), SystemProcess()]

    can_delete = [CommunityOwners(), SystemProcess()]

    can_search = [AnyUser(), SystemProcess()]

    can_search_user_communities = [AuthenticatedUser(), SystemProcess()]

    can_rename = [CommunityOwners(), SystemProcess()]



# -*- coding: utf-8 -*-
#
# Copyright (C) 2019 CERN.
# Copyright (C) 2019 Northwestern University.
#
# Invenio-RDM-Records is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Fake demo records."""

import json
import random


def create_fake_community(faker):
    """Create fake communities for demo purposes."""
    data_to_use = {
        "access": {
            "visibility": random.choice(["public", "restricted"]),
            "member_policy": random.choice(["open", "closed"]),
            "record_policy": random.choice(["open", "closed", "restricted"])
        },
        "id": faker.unique.domain_word(),
        "metadata": {
            "title": faker.sentence(nb_words=5, variable_nb_words=True),
            "description": faker.text(max_nb_chars=2000),
            "type": random.choice(
                ["organization", "event", "topic", "project"]),
            "curation_policy": faker.text(max_nb_chars=2000),
            "page": faker.text(max_nb_chars=2000),
            "website": "https://" + faker.domain_name(),  # fake.url()
            "funding": [
                {
                    "funder": {
                        "name": "European Commission",
                        "identifier": "03yrm5c26",
                        "scheme": "ror",
                    },
                    "award": {
                        "title": "OpenAIRE",
                        "number": "246686",
                        "identifier": "0000-0002-1825-0097",
                        "scheme": "orcid",
                    },
                }
            ],
            "organizations": [
                {
                    "name": "CERN",
                    "identifiers": [
                        {"identifier": "01ggx4157", "scheme": "ror"}],
                }
            ],
        },
    }

    return json.loads(json.dumps(data_to_use))



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2020 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""JS/CSS bundles for communities.

You include one of the bundles in a page like the example below (using
``comunities`` bundle as an example):

.. code-block:: html

    {{ webpack['communities.js']}}

"""

from __future__ import absolute_import, print_function

from invenio_assets.webpack import WebpackThemeBundle

communities = WebpackThemeBundle(
    __name__,
    'assets',
    default='semantic-ui',
    themes={
        'semantic-ui': dict(
            entry={
                'invenio-communities-theme':
                    './less/invenio_communities/theme.less',
                'invenio-communities-new':
                    './js/invenio_communities/new.js',
                'invenio-communities-privileges':
                    './js/invenio_communities/settings/privileges.js',
                'invenio-communities-profile':
                    './js/invenio_communities/settings/profile/index.js',
                'invenio-communities-frontpage':
                    './js/invenio_communities/frontpage.js',
                # 'invenio-communities-members':
                #     './js/invenio_communities/members.js',
                # 'invenio-communities-request':
                #     './js/invenio_communities/request.js',
                # 'invenio-communities-records':
                #     './js/invenio_communities/records.js',
                'invenio-communities-search':
                    './js/invenio_communities/search.js',
                # 'invenio-communities-records-search':
                #     './js/invenio_communities/records_search.js',
                # 'invenio-communities-records-curate':
                #     './js/invenio_communities/curate.js',
                # 'invenio-communities-collections-settings':
                #     './js/invenio_communities/collections/settings.js',
            },
            dependencies={
                'semantic-ui-css': '^2.4.1',
                'semantic-ui-react': '^0.88.2',
                '@ckeditor/ckeditor5-build-classic': '^16.0.0',
                '@ckeditor/ckeditor5-react': '^2.1.0',
                'axios': '^0.19.0',
                'formik': '^2.0.6',
                'lodash': '^4.17.15',
                'luxon': '^1.21.1',
                'path': '^0.12.7',
                'prop-types': '^15.7.2',
                'qs': '^6.9.1',
                'react': '^16.12.0',
                'react-dom': '^16.11.0',
                'react-redux': '^7.1.3',
                'react-searchkit': '^0.15.0',
                'redux': '^4.0.5',
                'redux-thunk': '^2.3.0',
                'yup': '^0.27.0',
            }
        ),
    }
)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2020 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Pytest configuration."""

import uuid

import pytest
from invenio_accounts.testutils import create_test_user
from invenio_indexer.api import RecordIndexer
from invenio_pidstore.models import PersistentIdentifier, PIDStatus

from invenio_communities.communities.records.api import Record


@pytest.fixture
def record_owner(db):
    """Record owner user."""
    yield create_test_user('record-owner@inveniosoftware.org')


@pytest.fixture
def record(db, es, record_owner):
    """Record fixture."""
    record = Record.create({
        'title': 'Title',
        '_owners': [record_owner.id],
    })
    recid = PersistentIdentifier.create(
        pid_type='recid', pid_value='12345',
        object_uuid=record.id, object_type='rec',
        status=PIDStatus.REGISTERED)
    db.session.commit()
    RecordIndexer().index_by_id(str(record.id))
    yield recid, record


# @pytest.fixture
# def accepted_community_record(db, es, community, record, record_owner):
#     """Community record fixture."""
#     comid, community = community
#     recid, record = record
#     # request = CommunityInclusionRequest.create(record_owner, id_=uuid.uuid4())
#     comm_record = CommunityRecord.create(record, recid, community, request)
#     comm_record.status = comm_record.Status.ACCEPTED
#     db.session.commit()
#     RecordIndexer().index_by_id(str(record.id))
#     yield comm_record



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 CERN.
#
# Invenio-Communities is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Command-line tools for demo module."""

import click
from faker import Faker
from flask.cli import with_appcontext

from .fixtures.demo import create_fake_community
from .fixtures.tasks import create_demo_community


@click.group()
def communities():
    """Invenio communities commands."""
    pass


@communities.command('demo')
@with_appcontext
def demo():
    """Create 100 fake communities for demo purposes."""
    click.secho('Creating demo communities...', fg='green')
    faker = Faker()
    for _ in range(100):
        fake_data = create_fake_community(faker)
        create_demo_community.delay(fake_data)

    click.secho('Created communities!', fg='green')



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2020 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Community module tests."""

import pytest
from flask import url_for
from invenio_accounts.testutils import login_user_via_session


@pytest.mark.skip()
def test_simple_flow(
        db, es_clear, community, record, client, record_owner,
        community_owner):
    """Test basics operations on records."""
    comid, community = community
    recid, record = record
    login_user_via_session(client, user=record_owner)
    community_records_list_url = url_for(
        'invenio_communities_records.community_records_list',
        pid_value=comid.pid_value)

    resp = client.post(community_records_list_url, json={
        'record_pid': recid.pid_value
    })
    assert resp.status_code == 201
    community_record_links = resp.json['links']
    community_record_id = resp.json['id']

    login_user_via_session(client, user=community_owner)
    resp = client.post(
        community_record_links['comment'],
        json={'message': 'Hello there'})
    assert resp.status_code == 201

    login_user_via_session(client, user=record_owner)
    resp = client.post(
        community_record_links['comment'],
        json={'message': 'Oh hi mark'})
    assert resp.status_code == 201

    login_user_via_session(client, user=community_owner)
    resp = client.get(community_record_links['self'])
    assert resp.status_code == 200

    resp = client.post(community_record_links['accept'])
    assert resp.status_code == 201

    login_user_via_session(client, user=record_owner)

    community_record_item_url = url_for(
        'invenio_communities_records.community_records_item',
        pid_value=comid.pid_value,
        community_record_id=community_record_id)

    resp = client.delete(community_record_item_url)
    assert resp.status_code == 204


@pytest.mark.skip()
def test_alternate_flow(
        db, es_clear, community, record, client, record_owner,
        community_owner):
    """Test basics operations on records."""
    comid, community = community
    recid, record = record
    login_user_via_session(client, user=community_owner)
    community_records_list_url = url_for(
        'invenio_communities_records.community_records_list',
        pid_value=comid.pid_value)

    resp = client.post(community_records_list_url, json={
        'record_pid': recid.pid_value
    })
    assert resp.status_code == 201
    community_record_links = resp.json['links']
    community_record_id = resp.json['id']

    login_user_via_session(client, user=record_owner)
    resp = client.post(
        community_record_links['comment'],
        json={'message': 'Hello there'})
    assert resp.status_code == 201

    login_user_via_session(client, user=community_owner)
    resp = client.post(
        community_record_links['comment'],
        json={'message': 'Oh hi mark'})
    assert resp.status_code == 201

    login_user_via_session(client, user=record_owner)
    resp = client.get(community_record_links['self'])
    assert resp.status_code == 200

    resp = client.post(community_record_links['accept'])
    assert resp.status_code == 201

    login_user_via_session(client, user=community_owner)

    community_record_item_url = url_for(
        'invenio_communities_records.community_records_item',
        pid_value=comid.pid_value,
        community_record_id=community_record_id)

    resp = client.delete(community_record_item_url)
    assert resp.status_code == 204




@pytest.mark.skip()
def test_records_permissions(db, es_clear, community, record, client, users):
    """Test permissions on records."""
    # TODO
    pass


@pytest.mark.skip()
def test_records_validation(db, es_clear, community, record, client, users):
    """Test records validation."""
    # TODO
    pass


@pytest.mark.skip()
def test_records_search(db, es_clear, community, record, client, users):
    """Test records search."""
    # TODO
    pass



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2020 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Community module tests."""

import pytest
from flask import url_for
from invenio_accounts.testutils import login_user_via_session


@pytest.mark.skip()
def test_simple_flow(db, es_clear, community, accepted_community_record, client, community_owner):
    """Test basic operations on collections."""
    comid, community = community

    collections_list_url = url_for(
        'invenio_communities_collections.collections_list',
        pid_value=comid.pid_value)

    # list
    resp = client.get(collections_list_url)
    assert resp.status_code == 200
    assert resp.json == {}

    # create
    collection_data = {
        'id': 'test',
        'title': 'Test collection',
        'description': 'Test collection description',
    }

    resp = client.post(collections_list_url, json=collection_data)
    assert resp.status_code == 401

    login_user_via_session(client, user=community_owner)
    resp = client.post(collections_list_url, json=collection_data)
    assert resp.status_code == 201
    created_resp_json = resp.json
    collection_item_url = created_resp_json['links']['self']
    assert created_resp_json == {
        'title': collection_data['title'],
        'description': collection_data['description'],
        'links': {
            'self': '/communities/{}/collections/test'.format(comid.pid_value)
        },
    }

    # read
    resp = client.get(collection_item_url)
    assert resp.status_code == 200
    assert resp.json == created_resp_json

    # update
    resp = client.put(collection_item_url, json={
        'title': 'New test title',
        # NOTE: removes description
    })
    assert resp.status_code == 200
    assert resp.json == {
        'title': 'New test title',
        'description': None,
        'links': {'self': collection_item_url},
    }

    # get record collections
    community_record_url = url_for(
        'invenio_communities_collections.records',
        pid_value=comid.pid_value,
        record_pid=accepted_community_record.record.pid.pid_value
    )
    resp = client.get(community_record_url)
    assert resp.status_code == 200
    assert '_collections' not in resp.json

    # update record collections
    resp = client.put(community_record_url, json={
        'collections': [{'id': 'test'}]
    })
    assert resp.status_code == 200
    assert resp.json['_collections'] == [{'id': 'test'}]

    # delete
    resp = client.delete(collection_item_url)
    assert resp.status_code == 204
    resp = client.get(collection_item_url)
    assert resp.status_code == 404


@pytest.mark.skip()
def test_permissions(db, es_clear, community, accepted_community_record, client, community_owner, authenticated_user, record_owner):
    """Test collection permissions."""
    # TODO: write tests for permissions
    pass



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 CERN.
# Copyright (C) 2021 TU Wien.
#
# Invenio-RDM-Records is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Routes for community-related pages provided by Invenio-Communities."""

from flask import current_app, render_template
from flask_login import login_required

from .decorators import pass_community, pass_community_logo, \
    require_community_owner

#
# Views
#

def communities_frontpage():
    """Communities index page."""
    return render_template(
        "invenio_communities/frontpage.html",
    )

def communities_search():
    """Communities search page."""
    return render_template(
        "invenio_communities/search.html",
    )

@login_required
def communities_new():
    """Communities creation page."""
    return render_template(
        "invenio_communities/new.html",
        form_config=dict(
            access=dict(
                visibilty=[
                    {
                        'text': 'Public',
                        'value': 'public',
                        'icon': 'group',
                        'helpText': 'Your community is publicly accessible ' \
                                    'and shows up in search results.'
                    },
                    {
                        'text': 'Restricted',
                        'value': 'restricted',
                        'icon': 'lock',
                        'helpText': 'Your community is restricted to users ' \
                                    'with access.'
                    }
                ]
            ),
            SITE_UI_URL=current_app.config["SITE_UI_URL"]
        ),
    )

@pass_community
@pass_community_logo
def communities_detail(community=None, logo=None, pid_value=None):
    """Community detail page."""
    return render_template(
        "invenio_communities/details/index.html",
        community=community.to_dict(), # TODO: use serializer
        logo=logo.to_dict() if logo else None,
        # TODO: inject this from a vocabulary in the community
        types={
            "organization": "Organization",
            "event": "Event",
            "topic": "Topic",
            "project": "Project"
        },
        # Pass permissions so we can disable partially UI components
        # e.g Settings tab
        permissions=community.has_permissions_to(['update']),
        active_menu_tab="search"
    )

@pass_community
@pass_community_logo
@require_community_owner
def communities_settings(community=None, logo=None, pid_value=None):
    """Community settings/profile page."""
    return render_template(
        "invenio_communities/details/settings/profile.html",
        community=community.to_dict(), # TODO: use serializer,
        logo=logo.to_dict() if logo else None,
        # TODO: inject this from a vocabulary in the community
        types={
            "organization": "Organization",
            "event": "Event",
            "topic": "Topic",
            "project": "Project"
        },
        # Pass permissions so we can disable partially UI components
        # e.g Settings tab
        permissions=community.has_permissions_to(['update']),
        active_menu_tab="settings"
    )

@pass_community
@require_community_owner
def communities_settings_privileges(community=None, pid_value=None):
    """Community settings/privileges page."""
    return render_template(
        "invenio_communities/details/settings/privileges.html",
        community=community.to_dict(), # TODO: use serializer,
        form_config=dict(
            access=dict(
                visibilty=[
                    {
                        'text': 'Public',
                        'value': 'public',
                        'icon': 'group',
                        'helpText': 'Your community is publicly accessible ' \
                                    'and shows up in search results.'
                    },
                    {
                        'text': 'Restricted',
                        'value': 'restricted',
                        'icon': 'lock',
                        'helpText': 'Your community is restricted to users ' \
                                    'with access.'
                    }
                ]
            ),
        ),
        # TODO: inject this from a vocabulary in the community
        types={
            "organization": "Organization",
            "event": "Event",
            "topic": "Topic",
            "project": "Project"
        },
        # Pass permissions so we can disable partially UI components
        # e.g Settings tab
        permissions=community.has_permissions_to(['update']),
        active_menu_tab="settings"
    )



#
# This file is part of Invenio.
# Copyright (C) 2016-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Create communities tables."""

import sqlalchemy as sa
import sqlalchemy_utils
from alembic import op
from sqlalchemy.dialects import mysql, postgresql

# revision identifiers, used by Alembic.
revision = "de9c14cbb0b2"
down_revision = "90642d415317"
branch_labels = ()
depends_on = '8ae99b034410'


def upgrade():
    """Upgrade database."""
    op.create_table(
        "communities_metadata_version",
        sa.Column(
            "created",
            sa.DateTime().with_variant(mysql.DATETIME(fsp=6), "mysql"),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column(
            "updated",
            sa.DateTime().with_variant(mysql.DATETIME(fsp=6), "mysql"),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column(
            "id",
            sqlalchemy_utils.types.uuid.UUIDType(),
            autoincrement=False,
            nullable=False,
        ),
        sa.Column(
            "json",
            sa.JSON()
            .with_variant(sqlalchemy_utils.types.json.JSONType(), "mysql")
            .with_variant(
                postgresql.JSONB(none_as_null=True, astext_type=sa.Text()),
                "postgresql"
            )
            .with_variant(sqlalchemy_utils.types.json.JSONType(), "sqlite"),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column(
            "version_id", sa.Integer(), autoincrement=False, nullable=True),
        sa.Column(
            "bucket_id",
            sqlalchemy_utils.types.uuid.UUIDType(),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column(
            "transaction_id",
            sa.BigInteger(),
            autoincrement=False,
            nullable=False
        ),
        sa.Column("end_transaction_id", sa.BigInteger(), nullable=True),
        sa.Column("operation_type", sa.SmallInteger(), nullable=False),
        sa.PrimaryKeyConstraint(
            "id",
            "transaction_id",
            name=op.f("pk_communities_metadata_version")
        ),
    )
    op.create_index(
        op.f("ix_communities_metadata_version_end_transaction_id"),
        "communities_metadata_version",
        ["end_transaction_id"],
        unique=False,
    )
    op.create_index(
        op.f("ix_communities_metadata_version_operation_type"),
        "communities_metadata_version",
        ["operation_type"],
        unique=False,
    )
    op.create_index(
        op.f("ix_communities_metadata_version_transaction_id"),
        "communities_metadata_version",
        ["transaction_id"],
        unique=False,
    )
    op.create_table(
        "communities_metadata",
        sa.Column(
            "created",
            sa.DateTime().with_variant(mysql.DATETIME(fsp=6), "mysql"),
            nullable=False,
        ),
        sa.Column(
            "updated",
            sa.DateTime().with_variant(mysql.DATETIME(fsp=6), "mysql"),
            nullable=False,
        ),
        sa.Column(
            "id",
            sqlalchemy_utils.types.uuid.UUIDType(),
            nullable=False
        ),
        sa.Column(
            "json",
            sa.JSON()
            .with_variant(sqlalchemy_utils.types.json.JSONType(), "mysql")
            .with_variant(
                postgresql.JSONB(none_as_null=True, astext_type=sa.Text()),
                "postgresql"
            )
            .with_variant(sqlalchemy_utils.types.json.JSONType(), "sqlite"),
            nullable=True,
        ),
        sa.Column("version_id", sa.Integer(), nullable=False),
        sa.Column(
            "bucket_id",
            sqlalchemy_utils.types.uuid.UUIDType(),
            nullable=True
        ),
        sa.ForeignKeyConstraint(
            ["bucket_id"],
            ["files_bucket.id"],
            name=op.f("fk_communities_metadata_bucket_id_files_bucket"),
        ),
        sa.PrimaryKeyConstraint("id", name=op.f("pk_communities_metadata")),
    )
    op.create_table(
        "communities_files",
        sa.Column(
            "created",
            sa.DateTime().with_variant(mysql.DATETIME(fsp=6), "mysql"),
            nullable=False,
        ),
        sa.Column(
            "updated",
            sa.DateTime().with_variant(mysql.DATETIME(fsp=6), "mysql"),
            nullable=False,
        ),
        sa.Column(
            "id",
            sqlalchemy_utils.types.uuid.UUIDType(),
            nullable=False
        ),
        sa.Column(
            "json",
            sa.JSON()
            .with_variant(sqlalchemy_utils.types.json.JSONType(), "mysql")
            .with_variant(
                postgresql.JSONB(none_as_null=True, astext_type=sa.Text()),
                "postgresql"
            )
            .with_variant(sqlalchemy_utils.types.json.JSONType(), "sqlite"),
            nullable=True,
        ),
        sa.Column("version_id", sa.Integer(), nullable=False),
        sa.Column(
            "key",
            sa.Text().with_variant(mysql.VARCHAR(length=255), "mysql"),
            nullable=False,
        ),
        sa.Column(
            "record_id",
            sqlalchemy_utils.types.uuid.UUIDType(),
            nullable=False
        ),
        sa.Column(
            "object_version_id",
            sqlalchemy_utils.types.uuid.UUIDType(),
            nullable=True,
        ),
        sa.ForeignKeyConstraint(
            ["object_version_id"],
            ["files_object.version_id"],
            name=op.f("fk_communities_files_object_version_id_files_object"),
            ondelete="RESTRICT",
        ),
        sa.ForeignKeyConstraint(
            ["record_id"],
            ["communities_metadata.id"],
            name=op.f("fk_communities_files_record_id_communities_metadata"),
            ondelete="RESTRICT",
        ),
        sa.PrimaryKeyConstraint("id", name=op.f("pk_communities_files")),
    )
    op.create_index(
        "uidx_communities_files_id_key",
        "communities_files", ["id", "key"],
        unique=True
    )


def downgrade():
    """Downgrade database."""
    op.drop_index(
        "uidx_communities_files_id_key", table_name="communities_files")
    op.drop_table("communities_files")
    op.drop_table("communities_metadata")
    op.drop_index(
        op.f("ix_communities_metadata_version_transaction_id"),
        table_name="communities_metadata_version",
    )
    op.drop_index(
        op.f("ix_communities_metadata_version_operation_type"),
        table_name="communities_metadata_version",
    )
    op.drop_index(
        op.f("ix_communities_metadata_version_end_transaction_id"),
        table_name="communities_metadata_version",
    )
    op.drop_table("communities_metadata_version")



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2020 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Invenio module that adds support for communities."""

from __future__ import absolute_import, print_function

import os

from setuptools import find_packages, setup

readme = open('README.rst').read()
history = open('CHANGES.rst').read()

tests_require = [
    'invenio-app>=1.3.0,<2.0.0',
    'pytest-invenio>=1.4.1,<2.0.0'
]

invenio_db_version = '>=1.0.9,<2.0.0'
invenio_search_version = '>=1.4.0,<2.0.0'

extras_require = {
    'docs': [
        'Sphinx>=3,<3.4.2',
    ],
    # Elasticsearch version
    'elasticsearch6': [
        f'invenio-search[elasticsearch6]{invenio_search_version}',
    ],
    'elasticsearch7': [
        f'invenio-search[elasticsearch7]{invenio_search_version}',
    ],
    # Databases
    'mysql': [
        f'invenio-db[mysql,versioning]{invenio_db_version}',
    ],
    'postgresql': [
        f'invenio-db[postgresql,versioning]{invenio_db_version}',
    ],
    'sqlite': [
        f'invenio-db[versioning]{invenio_db_version}',
    ],
    'tests': tests_require,
}

extras_require['all'] = []
for name, reqs in extras_require.items():
    if name in ('elasticsearch6', 'elasticsearch7',
                'mysql', 'postgresql', 'sqlite'):
        continue
    extras_require['all'].extend(reqs)

setup_requires = [
    'Babel>=1.3',
    'pytest-runner>=3.0.0,<5',
]

install_requires = [
    "flask-resources>=0.7.0,<0.8.0",
    "invenio-accounts>=1.4.3",
    "invenio-assets>=1.2.2",
    "invenio-base>=1.2.3",
    "invenio-files-rest>=1.2.0",
    "invenio-i18n>=1.2.0",
    "invenio-indexer>=1.2.0",
    "invenio-jsonschemas>=1.1.2",
    "invenio-mail>=1.0.2",
    "invenio-pidstore>=1.2.2",
    "invenio-records-permissions>=0.11.0,<0.12.0",
    "invenio-records>=1.5.0a4",
    "invenio-records-resources>=0.14.0",
    "invenio-rdm-records>=0.29.4",
    "uritemplate>=3.0.1",
]

packages = find_packages()


# Get the version string. Cannot be done with import!
g = {}
with open(os.path.join('invenio_communities', 'version.py'), 'rt') as fp:
    exec(fp.read(), g)
    version = g['__version__']

setup(
    name='invenio-communities',
    version=version,
    description=__doc__,
    long_description=readme + '\n\n' + history,
    keywords='invenio communities',
    license='MIT',
    author='CERN',
    author_email='info@inveniosoftware.org',
    url='https://github.com/inveniosoftware/invenio-communities',
    packages=packages,
    zip_safe=False,
    include_package_data=True,
    platforms='any',
    entry_points={
        'flask.commands': [
            'communities = invenio_communities.cli:communities',
        ],
        'invenio_base.apps': [
            'invenio_communities = invenio_communities:InvenioCommunities',
        ],
        'invenio_base.api_apps': [
            'invenio_communities = invenio_communities:InvenioCommunities',
        ],
        'invenio_base.blueprints': [
            'invenio_communities = invenio_communities.views:create_ui_blueprint',
            # 'invenio_communities_records = invenio_communities.communities.records.views:ui_blueprint',
            # 'invenio_communities_collections = invenio_communities.communities.records.collections.views:ui_blueprint',
        ],
        'invenio_base.api_blueprints': [
            'invenio_communities_api = invenio_communities.views:create_communities_api_blueprint',
        ],
        'invenio_db.models': [
            'invenio_communities = invenio_communities.communities.records.models',
            # 'invenio_communities_members = invenio_communities.members.models',
            # 'invenio_communities_records = invenio_communities.communities.records.models',
            # 'invenio_requests = invenio_communities.requests.models',
        ],
        'invenio_db.alembic': [
            'invenio_communities = invenio_communities:alembic',
        ],
        'invenio_search.mappings': [
            'communities = invenio_communities.communities.records.mappings',
        ],
        'invenio_jsonschemas.schemas': [
            'communities = invenio_communities.communities.records.jsonschemas',
        ],
        'invenio_assets.webpack': [
            'invenio_communities = invenio_communities.webpack:communities'
        ],
        'invenio_celery.tasks': [
            'invenio_communities = invenio_communities.fixtures.tasks',
        ],
    },
    extras_require=extras_require,
    install_requires=install_requires,
    setup_requires=setup_requires,
    tests_require=tests_require,
    classifiers=[
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Development Status :: 3 - Alpha",
     ],
)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2020-2021 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Invenio communities extension."""

from invenio_records_resources.services import FileService

from invenio_communities.communities import CommunityFileServiceConfig, \
    CommunityResource, CommunityResourceConfig, CommunityService, \
    CommunityServiceConfig

from . import config


class InvenioCommunities(object):
    """Invenio extension."""

    def __init__(self, app=None):
        """Extension initialization."""
        if app:
            self.init_app(app)

    def init_app(self, app):
        """Flask application initialization."""
        self.init_config(app)
        app.extensions['invenio-communities'] = self

        self.init_services(app)
        self.init_resource(app)

    def init_config(self, app):
        """Initialize configuration.

        Override configuration variables with the values in this package.
        """
        for k in dir(config):
            if k.startswith('COMMUNITIES_'):
                app.config.setdefault(k, getattr(config, k))

    def init_services(self, app):
        """Initialize communities service."""
        # Services
        self.service = CommunityService(
            CommunityServiceConfig,
            files_service=FileService(CommunityFileServiceConfig),
        )

    def init_resource(self, app):
        """Initialize communities resources."""
        # Resources
        self.communities_resource = CommunityResource(
            CommunityResourceConfig,
            self.service,
        )
